package ch.ethz.icr.tutorial.simpleipd;

import uchicago.src.sim.engine.SimInit;
import uchicago.src.sim.engine.SimpleModel;

/**
 * Simple Iterated Prisoners' Dilemma Model
 *
 * This is the first step of the tutorial sequence. This phase sets up a 2 x 2
 * iterated prisoner's dilemma game where the players can have one out of
 * our strategies.
 *
 * For the original model, see Cohen, Riolo, and Axelrod:
 * The Emergence of Social Organization in the Prisoner's Dilemma
 * (SFI Working Paper), 1999. http://www.santafe.edu/ (follow publications link).
 *
 * @author Luc Girardin
 * @author Lars-Erik Cederman
 * @author Laszlo Gulyas
 * @version 2.0
 */
public class Model extends SimpleModel {
    // Constants for agent strategy indices (see Cohen et al)
    static final int ALLC = 0;
    static final int TFT = 1;
    static final int ATFT = 2;
    static final int ALLD = 3;

    // The two players
    Player rowPlayer, colPlayer;

    // Model parameters
    long maxIter;                  // Number of iteration cycles

    /**
     * Initializing the model
     */
    public void setup() {
        // The setup() method of the parent MUST be called first!
        super.setup();

        // Setting the default model parameters
        maxIter = 4;

        // Setting the name of our model
        name = "SimpleIPD Model (Tutorial Step #1)";
    }

    /**
     * The method to build the Model's internals
     */
    public void buildModel() {
        // First we create two players with TFT and ALLD strategies, respectively...
        rowPlayer = new Player(TFT);
        colPlayer = new Player(ALLD);
        // ...then they have to get to know each other to be able to play
        rowPlayer.other = colPlayer;
        colPlayer.other = rowPlayer;

        // Making sure the simulation stops after 4 cycles
        setStoppingTime(maxIter);
    }

    /////////////////////////////////////////////////////////////////////////
    //  Iterated method  ////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////

    /**
     * The activities of the timestep
     */
    public void step() {
        // Get the current time
        int time = (int) getTickCount();
        rowPlayer.play(time);
        colPlayer.play(time);

        // Let the player's remember the other's action
        rowPlayer.remember();
        colPlayer.remember();

        // Let them update their accumulated payoff
        rowPlayer.addPayoff();
        colPlayer.addPayoff();

        // Reports the current moves to the screen
        System.out.println(getTickCount() + ":" +
                "rowPlayer=" + rowPlayer + " " +
                "colPlayer=" + colPlayer);
    }

    /////////////////////////////////////////////////////////////////////////
    //  Creating and starting your model  ///////////////////////////////////
    /////////////////////////////////////////////////////////////////////////

    /**
     * Entry point to the program
     *
     */
    public static void main(String[] args) {
        SimInit init = new SimInit();
        Model m = new Model();
        init.loadModel(m, null, false);
    }
}